# -*- coding: UTF-8 -*-
from mod.server.system.serverSystem import ServerSystem
from mod.server.serverEvent import ServerEvent
import mod.server.extraServerApi as serverApi


class Main(ServerSystem):

    def __init__(self, namespace, system_name):
        ServerSystem.__init__(self, namespace, system_name)
        namespace, system = serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName()
        self.ListenForEvent(namespace, system, ServerEvent.BlockRandomTickServerEvent, self, self.crop_grow)  # 监听事件
        self.ListenForEvent(namespace, system, ServerEvent.ServerBlockEntityTickEvent, self, self.torch_grow)  # 监听事件

    def crop_grow(self, event):
        x = event['posX']  # 获取坐标X
        y = event['posY']  # 获取坐标Y
        z = event['posZ']  # 获取坐标Z
        block_name = event['fullName']
        if block_name != 'design:custom_crop_0':  # 是否为自定义作物方块
            return
        dim_id = event['dimensionId']  # 获取维度ID
        block_comp = serverApi.GetEngineCompFactory().CreateBlockInfo(serverApi.GetLevelId())
        light_level = block_comp.GetBlockLightLevel((x, y, z), dim_id)  # 获取方块的光照程度
        if 5 < light_level <= 15:  # 光照程度如果在5~15之间
            block_data_comp = serverApi.GetEngineCompFactory().CreateBlockEntityData(serverApi.GetLevelId())
            block_data = block_data_comp.GetBlockEntityData(dim_id, (x, y, z))
            if not block_data:
                return
            growth = block_data['growth']  # 获得方块的生长值
            if not growth:
                growth = 0
                growth += 1
                block_data['growth'] = growth
            if growth >= 1:  # 生长值大于1时
                block_comp.SetBlockNew((x, y, z),
                                       {
                                           'name': 'minecraft:wheat',  # 转化为成熟的小麦
                                           'aux': 7
                                       })

    def torch_grow(self, event):
        x = event['posX']  # 获取坐标X
        y = event['posY']  # 获取坐标Y
        z = event['posZ']  # 获取坐标Z
        block_name = event['blockName']
        if block_name != 'design:custom_torch':  # 是否为自定义加速方块
            return
        dim_id = event['dimension']  # 获取维度ID
        block_comp = serverApi.GetEngineCompFactory().CreateBlockInfo(serverApi.GetLevelId())
        block_state_comp = serverApi.GetEngineCompFactory().CreateBlockState(serverApi.GetLevelId())
        block_entity_comp = serverApi.GetEngineCompFactory().CreateBlockEntityData(serverApi.GetLevelId())
        block_entity_data = block_entity_comp.GetBlockEntityData(dim_id, (x, y, z))  # 获取方块数据
        tick = block_entity_data['tick']  # 获取方块刷新次数
        if not tick:
            tick = 0
        tick += 1  # 加速火把方块会在每一帧自增一次刷新次数
        block_entity_data['tick'] = tick
        if tick % 20 != 0:  # 根据文档可知事件会在一秒刷新20次，当刷新次数与20求模为0时，即过了一秒钟
            return
        # 运用列表推导式，获得[(x - 1, y, z - 1), (x, y, z - 1), (x + 1, y, z - 1), (x + 1, y, z + 1), (x, y, z),
        # (x - 1, y, z + 1), (x, y, z + 1), (x - 1, y, z), (x + 1, y, z)]的坐标列表
        grow_poses = [(x + x_offset, y, z + z_offset) for x_offset in xrange(-1, 2) for z_offset in xrange(-1, 2)]
        grow_poses.remove((x, y, z))  # 由于坐标(x, y, z)是加速火把，所以移除这个坐标
        for grow_pos in grow_poses:
            block = block_comp.GetBlockNew(grow_pos, dim_id)
            if block and block['name'] == 'minecraft:wheat':  # 如果是小麦
                block_state = block_state_comp.GetBlockStates(grow_pos, dim_id)
                if block_state['growth'] < 7:  # 且小麦还未成熟，即成长值小于7
                    block_state['growth'] += 1  # 增加一级成长值
                    block_state_comp.SetBlockStates(grow_pos, block_state, dim_id)